<?php
/**
 * @package VirtueMart
 * @subpackage payment
 * @author CM-S.ru
 * @copyright Copyright (C) 2012-2015 CM-S.ru. All rights reserved.
 * @license GNU General Public License version 2 or later
 */

defined('_JEXEC') or die;

if (!class_exists('vmPSPlugin')) {
	require(JPATH_VM_PLUGINS.DS.'vmpsplugin.php');
}

class plgVMPaymentOnpay extends vmPSPlugin
{
	function __construct($subject, $config)
	{		
		parent::__construct($subject, $config);

		$jlang = JFactory::getLanguage();
		$jlang->load('plg_vmpayment_onpay', JPATH_ADMINISTRATOR, NULL, TRUE);
		
		$this->_loggable = true;
		$this->tableFields = array_keys($this->getTableSQLFields());
		$this->_tablepkey = 'id';
		$this->_tableId = 'id';
		$varsToPush = array(
			'merchant_id' => array('', 'char'),
			'secret_word' => array('', 'char'),
			'price_final' => array('', 'int'),
			'add_params' => array('', 'char'),
			'design' => array('', 'int'),
			'lang' => array('', 'char'),
			'payment_logos' => array('', 'char'),
			'status_pending' => array('', 'char'),
			'status_success' => array('', 'char'),
			'status_canceled' => array('', 'char'),
			'countries' => array('', 'char'),
			'min_amount' => array('', 'float'),
			'max_amount' => array('', 'float'),
			'cost_per_transaction' => array('', 'int'),
			'cost_percent_total' => array('', 'int'),
			'tax_id' => array(0, 'int')
		);
		
		$this->setConfigParameterable($this->_configTableFieldName, $varsToPush);
	}

    public function getVmPluginCreateTableSQL()
	{
		return $this->createTableSQL('Payment Onpay Table');
    }

	function getTableSQLFields()
	{
		$SQLfields = array(
			'id' => 'int(11) UNSIGNED NOT NULL AUTO_INCREMENT',
			'virtuemart_order_id' => 'int(1) UNSIGNED',
			'order_number' => 'char(64)',
			'virtuemart_paymentmethod_id' => 'mediumint(1) UNSIGNED',
			'payment_name' => 'varchar(5000)',
			'payment_order_total' => 'decimal(15,5) NOT NULL',
			'payment_currency' => 'smallint(1)',
			'cost_per_transaction' => 'decimal(10,2)',
			'cost_percent_total' => 'decimal(10,2)',
			'tax_id' => 'smallint(1)',
			'onpay_custom' => 'varchar(255)',
			'onpay_response_onpay_id' => 'varchar(100)',
			'onpay_response_pay_for' => 'varchar(100)',
			'onpay_response_order_amount' => 'decimal(10,2)',
			'onpay_response_order_currency' => 'char(3)',
			'onpay_response_balance_amount' => 'decimal(10,2)',
			'onpay_response_balance_currency' => 'char(3)',
			'onpay_response_exchange_rate' => 'decimal(10,2)',
			'onpay_response_paymentDateTime' => 'varchar(100)',
			'onpay_response_type' => 'varchar(100)',
			'onpay_response_md5' => 'varchar(40)',
			'onpay_response_raw' => 'varchar(512)'
		);

		return $SQLfields;
	}

	function plgVmConfirmedOrder($cart, $order)
	{
		if (!($method = $this->getVmPluginMethod($order['details']['BT']->virtuemart_paymentmethod_id))) {
	   		return null; // Another method was selected, do nothing
		}

		if (!$this->selectedThisElement($method->payment_element)) {
	    	return false;
		}
		
		$session = JFactory::getSession();
		$return_context = $session->getId();
		$this->_debug = $method->debug;
		$order_number = $order['details']['BT']->order_number;
		$this->logInfo('plgVmConfirmedOrder order number: '.$order_number, 'message');

		if (!class_exists('VirtueMartModelOrders')) {
			require(JPATH_VM_ADMINISTRATOR.DS.'models'.DS.'orders.php');
		}
		
		if (!class_exists('VirtueMartModelCurrency')) {
			require(JPATH_VM_ADMINISTRATOR.DS.'models'.DS.'currency.php');
		}

		if (!class_exists('TableVendors')) {
    		require(JPATH_VM_ADMINISTRATOR.DS.'table'.DS.'vendors.php');
		}
		
		$vendorModel = VmModel::getModel('Vendor');
		$vendorModel->setId(1);
		$vendor = $vendorModel->getVendor();
		$vendorModel->addImages($vendor, 1);
		$this->getPaymentCurrency($method);
		$currency_code_3 = shopFunctions::getCurrencyByID($method->payment_currency, 'currency_code_3');

		$paymentCurrency = CurrencyDisplay::getInstance($method->payment_currency);
		$totalInPaymentCurrency = round($paymentCurrency->convertCurrencyTo($method->payment_currency, $order['details']['BT']->order_total, false), 2);
		$cd = CurrencyDisplay::getInstance($cart->pricesCurrency);
		
		if ($totalInPaymentCurrency <= 0) {
			vmInfo(JText::_('PLG_ONPAY_VM2_ERROR_1'));
			return false;
		}

		$merchant_id = $this->_getMerchantID($method);
		$secret_word = $this->_getSecretWord($method);
		
		if (empty($merchant_id)) {
	    	vmInfo(JText::_('PLG_ONPAY_VM2_ERROR_2'));
	    	return false;
		}
		
		$virtuemart_paymentmethod_id = $order['details']['BT']->virtuemart_paymentmethod_id;
		
		$dbValues['order_number'] = $order_number;
		$dbValues['payment_name'] = $this->renderPluginName($method, $order);
		$dbValues['virtuemart_paymentmethod_id'] = $cart->virtuemart_paymentmethod_id;
		$dbValues['onpay_custom'] = $return_context;
		$dbValues['cost_per_transaction'] = $method->cost_per_transaction;
		$dbValues['cost_percent_total'] = $method->cost_percent_total;
		$dbValues['payment_currency'] = $method->payment_currency;
		$dbValues['payment_order_total'] = $totalInPaymentCurrency;
		$dbValues['tax_id'] = $method->tax_id;
		$this->storePSPluginInternalData($dbValues);
		
		$return_url = urlencode(substr(JURI::root(false, ''), 0, -1).'/pluginresponsereceived/'.$order_number.'/'.$virtuemart_paymentmethod_id);
		$fail_url = urlencode(substr(JURI::root(false, ''), 0, -1).'/pluginresponsecancel/'.$order_number.'/'.$virtuemart_paymentmethod_id);
		$price_final = $method->price_final ? 'true' : 'false';
		$add_params = trim($method->add_params, '&');
		$total_md5 = $this->to_float($totalInPaymentCurrency);
		$md5check = md5('fix;'.$total_md5.';'.$currency_code_3.';'.$order_number.';yes;'.$secret_word);
		$user_email = $order['details']['BT']->email;
		$user_phone = $order['details']['BT']->phone_1;
		$design = $method->design;
		$lang = '&ln='.JRequest::getVar('lang', 'ru');

		$url = 'http://secure.onpay.ru/pay/'.$merchant_id.'?f='.$design.'&pay_mode=fix&pay_for='.$order_number.'&price='.$totalInPaymentCurrency.'&ticker='.$currency_code_3.'&md5='.$md5check.'&url_success_enc='.$return_url.'&url_fail_enc='.$fail_url.'&price_final='.$price_final.'&user_phone='.$user_phone.'&user_email='.$user_email.$lang.'&'.$add_params; //формируем строку url
	
		$html = '<div style="margin: auto; text-align: center;">';
		$html .= '<form action="'.$url.'" method="post" name="vm_onpay_form">';
		$html .= '<h2>Подождите, идет перенаправление на Onpay.ru</h2>';
		$html .= '<input type="image" name="submit" src="http://onpay.ru/images/onpay_logo.gif" alt="onpay.ru" />';
		$html .= '</form></div>';
		$html .= '<script type="text/javascript">';
		$html .= 'document.vm_onpay_form.submit();';
		$html .= '</script>';
		
		$virtuemart_order_id = VirtueMartModelOrders::getOrderIdByOrderNumber($order_number);
		$modelOrder = VmModel::getModel('orders');
		$order['customer_notified'] = 1;
		$order['order_status'] = $method->status_pending;
		$modelOrder->updateStatusForOneOrder($virtuemart_order_id, $order, true);
		
		$cart->_confirmDone = false;
		$cart->_dataValidated = false;
		$cart->setCartIntoSession();
		JRequest::setVar('html', $html);
	}
	
	function plgVmgetPaymentCurrency($virtuemart_paymentmethod_id, $paymentCurrencyId)
	{
		if (!($method = $this->getVmPluginMethod($virtuemart_paymentmethod_id))) {
	    	return null; // Another method was selected, do nothing
		}
			
		if (!$this->selectedThisElement($method->payment_element)) {
	    	return false;
		}

		$this->getPaymentCurrency($method);
		$paymentCurrencyId = $method->payment_currency;
	}
	

    function plgVmOnPaymentResponseReceived($html)
	{
		if (!class_exists('VirtueMartCart')) {
			require(JPATH_VM_SITE.DS.'helpers'.DS.'cart.php');
		}
		
		if (!class_exists('shopFunctionsF')) {
			require(JPATH_VM_SITE.DS.'helpers'.DS.'shopfunctionsf.php');
		}
		
		if (!class_exists('VirtueMartModelOrders')) {
			require(JPATH_VM_ADMINISTRATOR.DS.'models'.DS.'orders.php');
		}
		
		$virtuemart_paymentmethod_id = JRequest::getInt('pm', 0);
		$order_number = JRequest::getString('on', 0);
		
		if (!($method = $this->getVmPluginMethod($virtuemart_paymentmethod_id))) {
	    	return null;
		}
		
		if (!$this->selectedThisElement($method->payment_element)) {
	    	return null;
		}
		
		if (!($virtuemart_order_id = VirtueMartModelOrders::getOrderIdByOrderNumber($order_number))) {
			return null;
		}
		
		if (!($paymentTable = $this->getDataByOrderId($virtuemart_order_id))) {
			return '';
		}
		
		$payment_name = $this->renderPluginName($method);
		$html = $this->_getPaymentResponseHtml($paymentTable, $payment_name);

		$cart = VirtueMartCart::getCart();
		$cart->emptyCart();
		
		return true;
	}

    function plgVmOnUserPaymentCancel()
	{
		if (!class_exists('VirtueMartModelOrders')) {
	    	require(JPATH_VM_ADMINISTRATOR.DS.'models'.DS.'orders.php');
		}
		
		$order_number = JRequest::getString('on', '');
		$virtuemart_paymentmethod_id = JRequest::getInt('pm', '');
		
		if (empty($order_number) or empty($virtuemart_paymentmethod_id) or !$this->selectedThisByMethodId($virtuemart_paymentmethod_id)) {
			return null;
		}
		
		if (!($virtuemart_order_id = VirtueMartModelOrders::getOrderIdByOrderNumber($order_number))) {
			return null;
		}
		
		if (!($paymentTable = $this->getDataByOrderId($virtuemart_order_id))) {
			return null;
		}

		VmInfo(JText::_('VMPAYMENT_PAYPAL_PAYMENT_CANCELLED'));
		
		$session = JFactory::getSession();
		$return_context = $session->getId();
		
		if (strcmp($paymentTable->onpay_custom, $return_context) === 0) {
			$this->handlePaymentUserCancel($virtuemart_order_id);
		}
		
		return true;
	}
	
	//функция выдает ответ для сервиса onpay в формате XML на чек запрос 
	function answer($type, $code, $pay_for, $order_amount, $order_currency, $text, $secret_word)
	{
		$md5 = strtoupper(md5("{$type};{$pay_for};{$order_amount};{$order_currency};{$code};".$secret_word)); 
		
		return "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<result>\n<code>{$code}</code>\n<pay_for>{$pay_for}</pay_for>\n<comment>{$text}</comment>\n<md5>{$md5}</md5>\n</result>";
	}
	
	//функция выдает ответ для сервиса onpay в формате XML на pay запрос 
	function answerpay($type, $code, $pay_for, $order_amount, $order_currency, $text, $onpay_id, $secret_word)
	{
		$md5 = strtoupper(md5("{$type};{$pay_for};{$onpay_id};{$pay_for};{$order_amount};{$order_currency};{$code};".$secret_word)); 
	
		return "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<result>\n<code>{$code}</code>\n<comment>{$text}</comment>\n<onpay_id>{$onpay_id}</onpay_id>\n<pay_for>{$pay_for}</pay_for>\n<order_id>{$pay_for}</order_id>\n<md5>{$md5}</md5>\n</result>"; 
	}
	
    function plgVmOnPaymentNotification()
	{
		if (!class_exists('VirtueMartModelOrders')) {
	    	require(JPATH_VM_ADMINISTRATOR.DS.'models'.DS.'orders.php');
		}
		
		$payment = JRequest::get('post');
		
		if (!isset($payment['pay_for'])) {
	    	return false;
		}
		
		$order_number = $payment['pay_for'];
		
		if (!($virtuemart_order_id = VirtueMartModelOrders::getOrderIdByOrderNumber($order_number))) {
	    	return false;
		}
		
		if (!($payments = $this->getDatasByOrderId($virtuemart_order_id))) {
	    	return false;
		}
		
		$method = $this->getVmPluginMethod($payments[0]->virtuemart_paymentmethod_id);

		if (!$this->selectedThisElement($method->payment_element)) {
	    	return false;
		}
		
		$this->_debug = $method->debug;
	
		$this->logInfo('onpay_data '.implode('   ', $payment), 'message');

		$secret_word = $this->_getSecretWord($method);
		$merchant_id = $this->_getMerchantID($method);
		
		$error = '';
		$type = $payment['type'];
		$order_amount = $this->to_float($payment['order_amount']);
		$pay_for = $order_number;
		$order_currency = $payment['order_currency'];
		
		//проверяем чек запрос 
		if ($type == 'check') { 
			//выдаем ответ OK на чек запрос 
			die("___rez___\n".$this->answer($type, 0, $pay_for, $order_amount, $order_currency, 'OK', $secret_word)); 
		}
		
		//проверяем pay запрос 
		if ($type == 'pay') {
			$order = array();
			$modelOrder = VmModel::getModel('orders');
			$order['customer_notified'] = 1;
			$order['order_status'] = $method->status_pending;
			
			//получаем данные
			$onpay_id = $payment['onpay_id']; 
			$balance_amount = $payment['balance_amount']; 
			$balance_currency = $payment['balance_currency']; 
			$exchange_rate = $payment['exchange_rate']; 
			$md5 = $payment['md5']; 
			
			//производим проверки входных данных 
			if (empty($onpay_id)) {
				$error .= JText::_('PLG_ONPAY_VM2_ERROR_3').'<br>';
			} else {
				if (!is_numeric(intval($onpay_id))) {
					$error .= JText::_('PLG_ONPAY_VM2_ERROR_4').'<br>';
				}
			}
			
			if (empty($order_amount)) {
				$error .= JText::_('PLG_ONPAY_VM2_ERROR_5').'<br>';
			} else {
				if (!is_numeric($order_amount)) {
					$error .= JText::_('PLG_ONPAY_VM2_ERROR_4').'<br>';
				}
			}
			 
			if (empty($balance_amount)) {
				$error .= JText::_('PLG_ONPAY_VM2_ERROR_5').'<br>';
			} else {
				if (!is_numeric(intval($balance_amount))) {
					$error .= JText::_('PLG_ONPAY_VM2_ERROR_4').'<br>';
				}
			}
			 
			if (empty($balance_currency)) {
				$error .= JText::_('PLG_ONPAY_VM2_ERROR_6').'<br>';
			} else {
				if (strlen($balance_currency) > 4) {
					$error .= JText::_('PLG_ONPAY_VM2_ERROR_7').'<br>';
				}
			}
			 
			if (empty($order_currency)) {
				$error .= JText::_('PLG_ONPAY_VM2_ERROR_6').'<br>';
			} else {
				if (strlen($order_currency) > 4) {
					$error .= JText::_('PLG_ONPAY_VM2_ERROR_7').'<br>';
				}
			}
			 
			if (empty($exchange_rate)) {
				$error .= JText::_('PLG_ONPAY_VM2_ERROR_5').'<br>';
			} else {
				if (!is_numeric($exchange_rate)) {
					$error .= JText::_('PLG_ONPAY_VM2_ERROR_4').'<br>';
				}
			} 

			//если нет ошибок 
			if (!$error) { 
				if ($pay_for) {
					//создаем строку хэша с присланных данных 
					$md5fb = strtoupper(md5($type.";".$pay_for.";".$onpay_id.";".$order_amount.";".$order_currency.";".$secret_word)); 
					//сверяем строчки хеша (присланную и созданную нами) 
					if ($md5fb != $md5) {
						$rezult = $this->answerpay($type, 8, $pay_for, $order_amount, $order_currency, JText::_('PLG_ONPAY_VM2_ERROR_8'), $onpay_id, $secret_word);
						$order['order_status'] = $method->status_canceled;
						$order['comments'] = JText::_('PLG_ONPAY_VM2_ERROR_8');
					} else { 
						$rezult = $this->answerpay($type, 0, $pay_for, $order_amount, $order_currency, 'OK', $onpay_id, $secret_word);
						$order['order_status'] = $method->status_success;
						$order['comments'] = sprintf(JText::_('PLG_ONPAY_VM2_COMM1'), $order_number);
					}
				} else {
					//если pay_for - не правильный формат 
					$rezult = $this->answerpay($type, 11, $pay_for, $order_amount, $order_currency, JText::_('PLG_ONPAY_VM2_ERROR_9'), $onpay_id, $secret_word); 
					$order['order_status'] = $method->status_canceled;
					$order['comments'] = JText::_('PLG_ONPAY_VM2_ERROR_9');
				} 
			} else {
				//если есть ошибки 
				$rezult = $this->answerpay($type, 12, $pay_for, $order_amount, $order_currency, JText::_('PLG_ONPAY_VM2_ERROR_9').': '.$error, $onpay_id, $secret_word);
				$order['order_status'] = $method->status_canceled;
				$order['comments'] = JText::_('PLG_ONPAY_VM2_ERROR_9').': '.$error;
			} 
		}
		
		$this->_storeOnpayInternalData($method, $payment, $virtuemart_order_id, $payments[0]->virtuemart_paymentmethod_id);
		$this->logInfo('plgVmOnPaymentNotification return new_status: '.$order['order_status'], 'message');

		$modelOrder->updateStatusForOneOrder($virtuemart_order_id, $order, true);

		if (isset($payments[0]->onpay_custom)) {
			$this->emptyCart($payments[0]->onpay_custom, $order_number);
		}
		
		die("___rez___\n".$rezult); //строка ___rez___ служит меткой для отделения заголовков от тела результата
	}
	
	function _storeOnpayInternalData($method, $payment, $virtuemart_order_id, $virtuemart_paymentmethod_id)
	{
		$db = JFactory::getDBO();
		$query = 'SHOW COLUMNS FROM `'.$this->_tablename.'`';
		$db->setQuery($query);
		$columns = $db->loadResultArray(0);
		
		$response_fields = array();
		$post_msg = '';
		foreach ($payment as $key => $value) {
			$post_msg .= $key.'='.$value.'<br />';
			$table_key = 'onpay_response_'.$key;
			
			if (in_array($table_key, $columns)) {
				$response_fields[$table_key] = $value;
			}
		}

		$response_fields['payment_name'] = $this->renderPluginName($method);
		$response_fields['onpay_response_raw'] = $post_msg;
		$response_fields['order_number'] = $payment['pay_for'];
		$response_fields['virtuemart_order_id'] = $virtuemart_order_id;
		$response_fields['virtuemart_paymentmethod_id'] = $virtuemart_paymentmethod_id;

		$this->storePSPluginInternalData($response_fields);
	}
	
	function plgVmOnShowOrderBEPayment($virtuemart_order_id, $payment_method_id)
	{
		if (!$this->selectedThisByMethodId($payment_method_id)) {
			return null;
		}

		if (!($payments = $this->getDatasByOrderId($virtuemart_order_id))) {
			return '';
		}

		$html = '<table class="adminlist" width="50%">'."\n";
		$html .= $this->getHtmlHeaderBE();
		$code = 'onpay_response_';
		
		foreach ($payments as $payment) {
			$html .= '<tr class="row1"><td>'.JText::_('PLG_ONPAY_VM2_DATE').'</td><td align="left">'.$payment->created_on.'</td></tr>';

			foreach ($payment as $key => $value) {
				if (substr($key, 0, strlen($code)) == $code) {
					$html .= $this->getHtmlRowBE($key, $value);
				}
			}
		}
		
		$html .= '</table>'."\n";
		
		return $html;
	}

	function _getMerchantID($method)
	{
		return $method->merchant_id;
	}
	
	function _getSecretWord($method)
	{
		return $method->secret_word;
    }
	
	function _getPaymentResponseHtml($paymentTable, $payment_name)
	{
		VmConfig::loadJLang('com_virtuemart');
		
		$html = '<table>'."\n";
		$html .= $this->getHtmlRow('PLG_ONPAY_VM2_PAYMENT_NAME', $payment_name);
		
		if (!empty($paymentTable)) {
	    	$html .= $this->getHtmlRow('PLG_ONPAY_VM2_ORDER_NUMBER', $paymentTable->order_number);
		}
		
		$html .= '</table>'."\n";

		return $html;
	}

    protected function checkConditions($cart, $method, $cart_prices)
	{
		$this->convert_condition_amount($method);
		$amount = $this->getCartAmount($cart_prices);
		$address = $cart->ST == 0 ? $cart->BT : $cart->ST;

		$amount_cond = ($amount >= $method->min_amount AND $amount <= $method->max_amount OR ($method->min_amount <= $amount AND ($method->max_amount == 0)));
		$countries = array();
	
		if (!empty($method->countries)) {
	    	if (!is_array($method->countries)) {
				$countries[0] = $method->countries;
			} else {
				$countries = $method->countries;
			}
		}

		if (!is_array($address)) {
	    	$address = array();
	    	$address['virtuemart_country_id'] = 0;
		}

		if (!isset($address['virtuemart_country_id'])) {
	    	$address['virtuemart_country_id'] = 0;
		}
		
		if (in_array($address['virtuemart_country_id'], $countries) || count($countries) == 0) {
	    	if ($amount_cond) {
				return true;
			}
		}

		return false;
	}
	
	//функция переводит число в нужный формат
	function to_float($sum)
	{ 
		$sum = round(floatval($sum), 2);
		$sum = sprintf('%01.2f', $sum);
		
		if (substr($sum, -1) == '0') {
			$sum = sprintf('%01.1f', $sum);
		}
		
		return $sum;
	}
	
	function getCosts(VirtueMartCart $cart, $method, $cart_prices)
	{
		if (preg_match('/%$/', $method->cost_percent_total)) {
	    	$cost_percent_total = substr($method->cost_percent_total, 0, -1);
		} else {
	    	$cost_percent_total = $method->cost_percent_total;
		}

		return $method->cost_per_transaction + ($cart_prices['salesPrice'] * $cost_percent_total * 0.01);
    }

	function plgVmOnStoreInstallPaymentPluginTable($jplugin_id)
	{
		return $this->onStoreInstallPluginTable($jplugin_id);
    }

    public function plgVmOnSelectCheckPayment(VirtueMartCart $cart)
	{
		return $this->OnSelectCheck($cart);
    }

    public function plgVmDisplayListFEPayment(VirtueMartCart $cart, $selected = 0, &$htmlIn)
	{
		return $this->displayListFE($cart, $selected, $htmlIn);
    }

    public function plgVmonSelectedCalculatePricePayment(VirtueMartCart $cart, array $cart_prices, $cart_prices_name)
	{
		return $this->onSelectedCalculatePrice($cart, $cart_prices, $cart_prices_name);
    }

    function plgVmOnCheckAutomaticSelectedPayment(VirtueMartCart $cart, array $cart_prices = array())
	{
		return $this->onCheckAutomaticSelected($cart, $cart_prices);
    }

    public function plgVmOnShowOrderFEPayment($virtuemart_order_id, $virtuemart_paymentmethod_id, $payment_name)
	{
		$this->onShowOrderFE($virtuemart_order_id, $virtuemart_paymentmethod_id, $payment_name);
    }

	function plgVmonShowOrderPrintPayment($order_number, $method_id)
	{
		return $this->onShowOrderPrint($order_number, $method_id);
    }

	function plgVmDeclarePluginParamsPaymentVM3(&$data)
	{
		return $this->declarePluginParams('payment', $data);
    }

    function plgVmSetOnTablePluginParamsPayment($name, $id, $table)
	{
		return $this->setOnTablePluginParams($name, $id, $table);
    }
}